/*
 * Copyright 1993-2006 NVIDIA Corporation.  All rights reserved.
 *
 * NOTICE TO USER:   
 *
 * This source code is subject to NVIDIA ownership rights under U.S. and 
 * international Copyright laws.  
 *
 * NVIDIA MAKES NO REPRESENTATION ABOUT THE SUITABILITY OF THIS SOURCE 
 * CODE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR 
 * IMPLIED WARRANTY OF ANY KIND.  NVIDIA DISCLAIMS ALL WARRANTIES WITH 
 * REGARD TO THIS SOURCE CODE, INCLUDING ALL IMPLIED WARRANTIES OF 
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.   
 * IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL, 
 * OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS 
 * OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE 
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE 
 * OR PERFORMANCE OF THIS SOURCE CODE.  
 *
 * U.S. Government End Users.  This source code is a "commercial item" as 
 * that term is defined at 48 C.F.R. 2.101 (OCT 1995), consisting  of 
 * "commercial computer software" and "commercial computer software 
 * documentation" as such terms are used in 48 C.F.R. 12.212 (SEPT 1995) 
 * and is provided to the U.S. Government only as a commercial end item.  
 * Consistent with 48 C.F.R.12.212 and 48 C.F.R. 227.7202-1 through 
 * 227.7202-4 (JUNE 1995), all U.S. Government End Users acquire the 
 * source code with only those rights set forth herein.
 */

/*
 * Slightly adapted 2009 by Ph. Robert wrt the Equalizer port
 */

#ifndef EQNBODY_RENDER_PARTICLES_H 
#define EQNBODY_RENDER_PARTICLES_H

struct GLEWContextStruct;
typedef struct GLEWContextStruct GLEWContext;

namespace eqNbody
{
    enum DisplayMode
    {
        PARTICLE_POINTS,
        PARTICLE_SPRITES,
        PARTICLE_SPRITES_COLOR,
        PARTICLE_NUM_MODES
    };

    class ParticleRenderer
        {
        public:
            ParticleRenderer( const GLEWContext* const glewContext ) 
                : _glewContext( glewContext )
                , m_pos(0)
                , m_numParticles(0)
                , m_pointSize(1.0f)
                , m_spriteSize(1.0f)
                , m_vertexShader(0)
                , m_pixelShader(0)
                , m_program(0)
                , m_texture(0)
                , m_pbo(0)
                , m_vboColor(0) { }

            ~ParticleRenderer();

            void init();

            void setPositions(float *pos, int numParticles);
            void setColors(float *color, int numParticles);
            void setPBO(unsigned int pbo, int numParticles);

            void draw(DisplayMode mode = PARTICLE_POINTS);
            
            void setPointSize(float size)  { m_pointSize = size; }
            void setSpriteSize(float size) { m_spriteSize = size; }
            const GLEWContext* glewGetContext() const { return _glewContext; }

        protected: // methods
            void _createTexture(int resolution);
            void _drawPoints(bool color = false);
        private:
            const GLEWContext* _glewContext;
        protected: // data
            float *m_pos;
            int m_numParticles;
            
            float m_pointSize;
            float m_spriteSize;
            
            unsigned int m_vertexShader;
            unsigned int m_pixelShader;
            unsigned int m_program;
            unsigned int m_texture;
            unsigned int m_pbo;
            unsigned int m_vboColor;
        };
}
#endif // EQNBODY_RENDER_PARTICLES_H
